@@ -5,6 +5,7 @@ module Test.Tasty.Ingredients.ConsoleReporter
   ( consoleTestReporter
   , Quiet(..)
   , HideSuccesses(..)
+  , AnsiTricks(..)
   -- * Internals
   -- | The following functions and datatypes are internals that are exposed to
   -- simplify the task of rolling your own custom console reporter UI.
@@ -14,6 +15,7 @@ module Test.Tasty.Ingredients.ConsoleReporter
   , useColor
   -- ** Test failure statistics
   , Statistics(..)
+  , computeStatistics
   , printStatistics
   , printStatisticsNoTime
   -- ** Outputting results
@@ -286,6 +288,9 @@ instance Semigroup Statistics where
   (<>) = mappend
 #endif
 
+-- | @computeStatistics@ computes a summary 'Statistics' for
+-- a given state of the 'StatusMap'.
+-- Useful in combination with @printStatistics@
 computeStatistics :: StatusMap -> IO Statistics
 computeStatistics = getApp . foldMap (\var -> Ap $
   (\r -> Statistics 1 (if resultSuccessful r then 0 else 1))
@@ -381,6 +386,7 @@ consoleTestReporter =
     [ Option (Proxy :: Proxy Quiet)
     , Option (Proxy :: Proxy HideSuccesses)
     , Option (Proxy :: Proxy UseColor)
+    , Option (Proxy :: Proxy AnsiTricks)
     ] $
   \opts tree -> Just $ \smap -> do
 
@@ -389,6 +395,7 @@ consoleTestReporter =
     Quiet quiet = lookupOption opts
     HideSuccesses hideSuccesses = lookupOption opts
     NumThreads numThreads = lookupOption opts
+    AnsiTricks ansiTricks = lookupOption opts
 
   if quiet
     then do
@@ -398,6 +405,7 @@ consoleTestReporter =
 
       do
       isTerm <- hSupportsANSI stdout
+      isTermColor <- hSupportsANSIColor stdout
 
       (\k -> if isTerm
         then (do hideCursor; k) `finally` showCursor
@@ -406,15 +414,15 @@ consoleTestReporter =
           hSetBuffering stdout LineBuffering
 
           let
-            ?colors = useColor whenColor isTerm
+            ?colors = useColor whenColor isTermColor
 
           let
             toutput = buildTestOutput opts tree
 
           case () of { _
-            | hideSuccesses && isTerm ->
+            | hideSuccesses && isTerm && ansiTricks ->
                 consoleOutputHidingSuccesses toutput smap
-            | hideSuccesses && not isTerm ->
+            | hideSuccesses ->
                 streamOutputHidingSuccesses toutput smap
             | otherwise -> consoleOutput toutput smap
           }
@@ -461,6 +469,32 @@ instance IsOption UseColor where
   optionHelp = return "When to use colored output (default: 'auto')"
   optionCLParser = mkOptionCLParser $ metavar "never|always|auto"
 
+-- | By default, when the option @--hide-successes@ is given and the output
+-- goes to an ANSI-capable terminal, we employ some ANSI terminal tricks to
+-- display the name of the currently running test and then erase it if it
+-- succeeds.
+--
+-- These tricks sometimes fail, however—in particular, when the test names
+-- happen to be longer than the width of the terminal window. See
+--
+-- * <https://github.com/feuerbach/tasty/issues/152>
+--
+-- * <https://github.com/feuerbach/tasty/issues/250>
+--
+-- When that happens, this option can be used to disable the tricks. In
+-- that case, the test name will be printed only once the test fails.
+newtype AnsiTricks = AnsiTricks Bool
+  deriving Typeable
+
+instance IsOption AnsiTricks where
+  defaultValue = AnsiTricks True
+  parseValue = fmap AnsiTricks . safeReadBool
+  optionName = return "ansi-tricks"
+  optionHelp = return $
+    -- Multiline literals don't work because of -XCPP.
+    "Enable various ANSI terminal tricks. " ++
+    "Can be set to 'true' (default) or 'false'."
+
 -- | @useColor when isTerm@ decides if colors should be used,
 --   where @isTerm@ indicates whether @stdout@ is a terminal device.
 --
