@@ -4,8 +4,11 @@
 
 module Test.Tasty.Patterns
   ( TestPattern(..)
+  , parseExpr
   , parseTestPattern
   , noPattern
+  , Path
+  , exprMatches
   , testPatternMatches
   ) where
 
@@ -14,11 +17,12 @@ import Test.Tasty.Patterns.Types
 import Test.Tasty.Patterns.Parser
 import Test.Tasty.Patterns.Eval
 
-import Data.Monoid
 import Data.Char
-import qualified Data.Sequence as Seq
 import Data.Typeable
 import Options.Applicative hiding (Success)
+#if !MIN_VERSION_base(4,11,0)
+import Data.Monoid
+#endif
 
 newtype TestPattern = TestPattern (Maybe Expr)
   deriving (Typeable, Show, Eq)
@@ -33,18 +37,25 @@ instance IsOption TestPattern where
   optionHelp = return "Select only tests which satisfy a pattern or awk expression"
   optionCLParser = mkOptionCLParser (short 'p' <> metavar "PATTERN")
 
+parseExpr :: String -> Maybe Expr
+parseExpr s
+  | all (\c -> isAlphaNum c || c `elem` "._- ") s =
+    Just $ ERE s
+  | otherwise = parseAwkExpr s
+
 parseTestPattern :: String -> Maybe TestPattern
 parseTestPattern s
   | null s = Just noPattern
-  | all (\c -> isAlphaNum c || c `elem` "._- ") s =
-    Just . TestPattern . Just $ ERE s
-  | otherwise = TestPattern . Just <$> parseAwkExpr s
+  | otherwise = TestPattern . Just <$> parseExpr s
 
-testPatternMatches :: TestPattern -> Seq.Seq String -> Bool
+exprMatches :: Expr -> Path -> Bool
+exprMatches e fields =
+  case withFields fields $ asB =<< eval e of
+    Left msg -> error msg
+    Right b -> b
+
+testPatternMatches :: TestPattern -> Path -> Bool
 testPatternMatches pat fields =
   case pat of
     TestPattern Nothing -> True
-    TestPattern (Just e) ->
-      case withFields fields $ asB =<< eval e of
-        Left msg -> error msg
-        Right b -> b
+    TestPattern (Just e) -> exprMatches e fields
